package com.clx.performance.service.impl.vehiclewarn;

import com.clx.performance.dao.OrderGoodsDao;
import com.clx.performance.dao.impl.vehiclewarn.VehicleWarnRangeDaoImpl;
import com.clx.performance.dao.vehiclewarn.VehicleWarnChildDao;
import com.clx.performance.dao.vehiclewarn.VehicleWarnConfigDao;
import com.clx.performance.dao.vehiclewarn.VehicleWarnInfoDao;
import com.clx.performance.dto.gd.GdRouteDTO;
import com.clx.performance.enums.OrderChildEnum;
import com.clx.performance.enums.ResultEnum;
import com.clx.performance.enums.vehiclewarn.VehicleWarnConfigEnum;
import com.clx.performance.enums.vehiclewarn.VehicleWarnInfoEnum;
import com.clx.performance.enums.vehiclewarn.VehicleWarnRangEnum;
import com.clx.performance.model.OrderChild;
import com.clx.performance.model.OrderGoods;
import com.clx.performance.model.vehiclewarn.VehicleWarnChild;
import com.clx.performance.model.vehiclewarn.VehicleWarnConfig;
import com.clx.performance.model.vehiclewarn.VehicleWarnInfo;
import com.clx.performance.model.vehiclewarn.VehicleWarnRang;
import com.clx.performance.service.trace.TruckTraceService;
import com.clx.performance.service.vehiclewarn.VehicleWarnMqService;
import com.clx.performance.service.vehiclewarn.VehicleWarnReceiveAddressWarnService;
import com.clx.performance.utils.gd.GdService;
import com.msl.common.exception.ServiceSystemException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

@Slf4j
@Service
public class VehicleWarnReceiveAddressWarnServiceImpl implements VehicleWarnReceiveAddressWarnService {
    private static final Integer MIN_FER_HOUR = 60;
    private static final Integer SEC_FER_MIN = 60;
    private static final Integer DISTANCE_DEFAULT = 2000;
    @Autowired
    private OrderGoodsDao orderGoodsDao;
    @Autowired
    private VehicleWarnChildDao vehicleWarnChildDao;
    @Autowired
    private VehicleWarnConfigDao vehicleWarnConfigDao;
    @Autowired
    private VehicleWarnInfoDao vehicleWarnInfoDao;
    @Autowired
    private VehicleWarnRangeDaoImpl vehicleWarnRangeDao;
    @Autowired
    private GdService gdService;
    @Autowired
    private TruckTraceService truckTraceService;

    @Autowired
    private VehicleWarnMqService vehicleWarnMqService;


    /**
     * 前往货源地超时
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void gotoReceiveAddressTimeout(OrderChild orderChild, VehicleWarnChild vehicleWarnChild){
        if (vehicleWarnChild.getArriveReceiveExpectTime() == null) {return;}

        // 未装车前往目的地判断
        if (Objects.equals(orderChild.getStatus(), OrderChildEnum.Status.ARRIVE_SEND.getCode())){

            // 首次不处理预警
            if (vehicleWarnChild.getReceiveAddressDistance() == null){
                return;
            }
            else {
                int needDistance = getNeedDistance(vehicleWarnChild.getTruckNo(), vehicleWarnChild.getDriverUserNo(),
                        vehicleWarnChild.getReceiveLongitude(), vehicleWarnChild.getReceiveLatitude());

                // 未前往目的地不处理预警
                if (vehicleWarnChild.getReceiveAddressDistance()-needDistance < DISTANCE_DEFAULT){return;}

                // 假定当前已装车
                orderChild.setLoadTime(LocalDateTime.now());
            }
        }

        Integer expectTime = vehicleWarnChild.getArriveReceiveExpectTime();

        // 计算需要的时间
        Integer needTime = getNeedTime(vehicleWarnChild.getTruckNo(), vehicleWarnChild.getDriverUserNo(),
                vehicleWarnChild.getReceiveLongitude(), vehicleWarnChild.getReceiveLatitude());

        List<VehicleWarnConfig> vehicleWarnConfigList = vehicleWarnConfigDao.listByWarnType(VehicleWarnConfigEnum.WarnType.GOTO_RECEIVE_TIMEOUT.getCode());
        if (vehicleWarnConfigList.isEmpty()) {return;}

        for (VehicleWarnConfig item : vehicleWarnConfigList) {
            doGotoReceiveAddressTimeout(orderChild, vehicleWarnChild, item, expectTime, needTime);
        }

    }

    private void doGotoReceiveAddressTimeout(OrderChild orderChild, VehicleWarnChild vehicleWarnChild, VehicleWarnConfig vehicleWarnConfig, Integer expectTime, Integer needTime){
        VehicleWarnInfo vehicleWarnInfo = vehicleWarnInfoDao.findEnableByChildNoAndWarnConfigType(vehicleWarnChild.getChildNo(), vehicleWarnConfig.getId()).orNull();

        // 暂停检测
        boolean suspend = suspendCheck(vehicleWarnChild, vehicleWarnConfig.getId());

        // 已暂停
        if (vehicleWarnInfo != null && Objects.equals(vehicleWarnInfo.getStatus(), VehicleWarnInfoEnum.Status.SUSPEND.getCode())){
            if (suspend) {return;}

            // 暂停恢复
            vehicleWarnInfo.setStatus(VehicleWarnInfoEnum.Status.INIT.getCode());
            vehicleWarnInfoDao.updateStatus(vehicleWarnInfo);
        }

        LocalDateTime loadTime = orderChild.getLoadTime();

        // 超时判断
        int time = vehicleWarnConfig.getTimeoutRatio().multiply(new BigDecimal(expectTime)).divide(new BigDecimal("100"), 0, RoundingMode.HALF_UP).intValue();
        if (loadTime.plusMinutes(expectTime).plusMinutes(time).isAfter(LocalDateTime.now().plusMinutes(needTime))){return;}

        // 更新
        if (vehicleWarnInfo != null && Objects.equals(vehicleWarnInfo.getStatus(), VehicleWarnInfoEnum.Status.INIT.getCode())){

            // 计算下次生效时间
            LocalDateTime nextTime = vehicleWarnInfo.getWarnTime().plusMinutes(vehicleWarnConfig.getInterval().multiply(new BigDecimal(MIN_FER_HOUR)).intValue());
            if (nextTime.isAfter(LocalDateTime.now())) {return;}

            vehicleWarnInfo.setWarnTime(LocalDateTime.now());
            vehicleWarnInfoDao.updateWarn(vehicleWarnInfo);
            return;
        }

        // 新增
        vehicleWarnInfo = new VehicleWarnInfo();
        vehicleWarnInfo.setWarnConfigId(vehicleWarnConfig.getId());
        vehicleWarnInfo.setWarnType(vehicleWarnConfig.getWarnType());
        vehicleWarnInfo.setWarnName(vehicleWarnConfig.getWarnName());
        vehicleWarnInfo.setWarnName(vehicleWarnConfig.getWarnName());
        vehicleWarnInfo.setWarnLevel(vehicleWarnConfig.getWarnLevel());

        vehicleWarnInfo.setChildNo(orderChild.getChildNo());
        vehicleWarnInfo.setSendAddressId(orderChild.getSendAddressId());
        vehicleWarnInfo.setSendAddress(orderChild.getSendAddress());
        vehicleWarnInfo.setReceiveAddressId(orderChild.getReceiveAddressId());
        vehicleWarnInfo.setReceiveAddress(orderChild.getReceiveAddress());

        vehicleWarnInfo.setTruckId(orderChild.getTruckId());
        vehicleWarnInfo.setTruckNo(orderChild.getTruckNo());
        vehicleWarnInfo.setDriverUserNo(orderChild.getDriverUserNo());
        vehicleWarnInfo.setDriverName(orderChild.getDriverName());
        vehicleWarnInfo.setDriverMobile(orderChild.getDriverMobile());

        vehicleWarnInfo.setTakeTime(orderChild.getPayTime());
        vehicleWarnInfo.setWarnTime(LocalDateTime.now());

        vehicleWarnInfoDao.saveEntity(vehicleWarnInfo);
        // 发送mq （预警通知）
        vehicleWarnMqService.truckWarnNotice(vehicleWarnInfo.getId());
    }

    /**
     * 到达货源地超时
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void arriveReceiveAddressTimeout(OrderChild orderChild, VehicleWarnChild vehicleWarnChild){
        if (vehicleWarnChild.getArriveReceiveExpectTime() == null) {return;}

        // 未装车前往目的地判断
        if (Objects.equals(orderChild.getStatus(), OrderChildEnum.Status.ARRIVE_SEND.getCode())){
            int needDistance = getNeedDistance(vehicleWarnChild.getTruckNo(), vehicleWarnChild.getDriverUserNo(),
                    vehicleWarnChild.getReceiveLongitude(), vehicleWarnChild.getReceiveLatitude());

            // 首次不处理预警
            if (vehicleWarnChild.getReceiveAddressDistance() == null){
                vehicleWarnChild.setReceiveAddressDistance(needDistance);
                vehicleWarnChildDao.updateReceiveAddressDistance(vehicleWarnChild);
                return;
            }
            else {
                // 未前往目的地不处理预警
                if (vehicleWarnChild.getReceiveAddressDistance()-needDistance < DISTANCE_DEFAULT){return;}

                vehicleWarnChild.setReceiveAddressDistance(needDistance);
                vehicleWarnChildDao.updateReceiveAddressDistance(vehicleWarnChild);
            }
        }

        OrderGoods orderGoods = orderGoodsDao.getByOrderGoodsNo(orderChild.getOrderGoodsNo()).orElseThrow(ResultEnum.DATA_NOT_FIND);

        // 计算需要的时间
        int needTime = getNeedTime(vehicleWarnChild.getTruckNo(), vehicleWarnChild.getDriverUserNo(),
                vehicleWarnChild.getReceiveLongitude(), vehicleWarnChild.getReceiveLatitude());

        // 超时判断
        LocalDateTime now = LocalDateTime.now();
        if (orderGoods.getLastArriveReceiveTime().isAfter(now.plusMinutes(needTime))) {return;}

        List<VehicleWarnConfig> vehicleWarnConfigList = vehicleWarnConfigDao.listByWarnType(VehicleWarnConfigEnum.WarnType.ARRIVE_RECEIVE_DELAY.getCode());
        if (vehicleWarnConfigList.isEmpty()) {return;}

        for (VehicleWarnConfig item : vehicleWarnConfigList) {
            doArriveReceiveAddressTimeout(orderChild, vehicleWarnChild, item);
        }

    }

    private void doArriveReceiveAddressTimeout(OrderChild orderChild, VehicleWarnChild vehicleWarnChild, VehicleWarnConfig vehicleWarnConfig){
        VehicleWarnInfo vehicleWarnInfo = vehicleWarnInfoDao.findEnableByChildNoAndWarnConfigType(vehicleWarnChild.getChildNo(), vehicleWarnConfig.getId()).orNull();

        // 暂停检测
        boolean suspend = suspendCheck(vehicleWarnChild, vehicleWarnConfig.getId());

        // 已暂停
        if (vehicleWarnInfo != null && Objects.equals(vehicleWarnInfo.getStatus(), VehicleWarnInfoEnum.Status.SUSPEND.getCode())){
            if (suspend) {return;}

            // 暂停恢复
            vehicleWarnInfo.setStatus(VehicleWarnInfoEnum.Status.INIT.getCode());
            vehicleWarnInfoDao.updateStatus(vehicleWarnInfo);
        }

        // 更新
        if (vehicleWarnInfo != null && Objects.equals(vehicleWarnInfo.getStatus(), VehicleWarnInfoEnum.Status.INIT.getCode())){

            // 计算下次生效时间
            LocalDateTime nextTime = vehicleWarnInfo.getWarnTime().plusMinutes(vehicleWarnConfig.getInterval().multiply(new BigDecimal(MIN_FER_HOUR)).intValue());
            if (nextTime.isAfter(LocalDateTime.now())) {return;}

            vehicleWarnInfo.setWarnTime(LocalDateTime.now());
            vehicleWarnInfoDao.updateWarn(vehicleWarnInfo);
            return;
        }

        // 新增
        vehicleWarnInfo = new VehicleWarnInfo();
        vehicleWarnInfo.setWarnConfigId(vehicleWarnConfig.getId());
        vehicleWarnInfo.setWarnType(vehicleWarnConfig.getWarnType());
        vehicleWarnInfo.setWarnName(vehicleWarnConfig.getWarnName());
        vehicleWarnInfo.setWarnLevel(vehicleWarnConfig.getWarnLevel());

        vehicleWarnInfo.setChildNo(orderChild.getChildNo());
        vehicleWarnInfo.setSendAddressId(orderChild.getSendAddressId());
        vehicleWarnInfo.setSendAddress(orderChild.getSendAddress());
        vehicleWarnInfo.setReceiveAddressId(orderChild.getReceiveAddressId());
        vehicleWarnInfo.setReceiveAddress(orderChild.getReceiveAddress());

        vehicleWarnInfo.setTruckId(orderChild.getTruckId());
        vehicleWarnInfo.setTruckNo(orderChild.getTruckNo());
        vehicleWarnInfo.setDriverUserNo(orderChild.getDriverUserNo());
        vehicleWarnInfo.setDriverName(orderChild.getDriverName());
        vehicleWarnInfo.setDriverMobile(orderChild.getDriverMobile());

        vehicleWarnInfo.setTakeTime(orderChild.getPayTime());
        vehicleWarnInfo.setWarnTime(LocalDateTime.now());

        vehicleWarnInfoDao.saveEntity(vehicleWarnInfo);

        // 发送mq （预警通知）
        vehicleWarnMqService.truckWarnNotice(vehicleWarnInfo.getId());
    }


    /**
     * 暂停检测
     */
    private boolean suspendCheck(VehicleWarnChild vehicleWarnChild, Integer warnConfigId){
        // 报警暂停
        List<VehicleWarnRang> vehicleWarnRangList = vehicleWarnRangeDao.ListByWarnConfigId(warnConfigId);
        if (!vehicleWarnRangList.isEmpty()){
            for (VehicleWarnRang item : vehicleWarnRangList) {
                if (Objects.equals(item.getWarnRang(), VehicleWarnRangEnum.WarnRange.ALL.getCode())){
                    return true;
                }
                if (Objects.equals(item.getWarnRang(), VehicleWarnRangEnum.WarnRange.ORDER.getCode())){
                    if (Objects.equals(vehicleWarnChild.getOrderNo(), item.getOrderNo())) {
                        return true;
                    }
                }
                if (Objects.equals(item.getWarnRang(), VehicleWarnRangEnum.WarnRange.CHILD.getCode())){
                    if (Objects.equals(vehicleWarnChild.getChildNo(), item.getChildNo())) {
                        return true;
                    }
                }
            }
        }

        return false;
    }

    /**
     * 获取需要时间（min）
     */
    private int getNeedTime(String truckNo, Long userNo,
                                 BigDecimal destinationLongitude, BigDecimal destinationLatitude){
        int needTime = 0;
        BigDecimal[] lastPosition = truckTraceService.getLastPosition(truckNo, userNo);
        if (lastPosition != null) {
            try {
                List<GdRouteDTO> gdRouteDTOS = gdService.getRoute(lastPosition[0], lastPosition[1], destinationLongitude, destinationLatitude);

                if (!gdRouteDTOS.isEmpty()){
                    needTime = gdRouteDTOS.get(0).getDuration() /SEC_FER_MIN;
                }

            } catch (ServiceSystemException e) {
                log.info("获取高德线路失败, truckNo:{}, userNo:{}, msg:{}", truckNo, userNo, e.getMessage());
            }
        }

        return needTime;
    }

    /**
     * 获取需要距离（m）
     */
    private int getNeedDistance(String truckNo, Long userNo,
                                 BigDecimal destinationLongitude, BigDecimal destinationLatitude){
        int needDistance = 0;
        BigDecimal[] lastPosition = truckTraceService.getLastPosition(truckNo, userNo);
        if (lastPosition != null) {
            try {
                List<GdRouteDTO> gdRouteDTOS = gdService.getRoute(lastPosition[0], lastPosition[1], destinationLongitude, destinationLatitude);

                if (!gdRouteDTOS.isEmpty()){
                    needDistance = gdRouteDTOS.get(0).getDistance();
                }

            } catch (ServiceSystemException e) {
                log.info("获取高德线路失败, truckNo:{}, userNo:{}, msg:{}", truckNo, userNo, e.getMessage());
            }
        }

        return needDistance;
    }

}
